package com.weilab.biology.controller;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.weilab.biology.core.data.dto.AppConfigDto;
import com.weilab.biology.core.data.dto.JobDto;
import com.weilab.biology.core.data.enums.JobStatusEnum;
import com.weilab.biology.core.data.po.Job;
import com.weilab.biology.core.data.vo.result.CommonError;
import com.weilab.biology.core.data.vo.result.Result;
import com.weilab.biology.core.data.vo.result.error.JobError;
import com.weilab.biology.core.validation.EnumValidation;
import com.weilab.biology.mapper.JobMapper;
import com.weilab.biology.service.AppConfigService;
import com.weilab.biology.service.JobService;
import com.weilab.biology.util.JobIdUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;


@RestController
@RequestMapping("job")
public class JobController {

    @Autowired
    private JobService jobService;

    @Autowired
    private AppConfigService appConfigService;

    @Autowired
    private JobMapper jobMapper;

    @PostMapping("/submit")
    public Result submit(@RequestParam String appName,
                         @RequestParam(required = false) String jobId,
                         @RequestParam(required = false) String dataStr,
                         @RequestParam(required = false) MultipartFile dataFile,
                         @RequestParam(defaultValue = "{}") String param,
                         @RequestParam String mail,
                         @RequestParam(defaultValue = "0") Integer type) {
        if (dataFile == null && StrUtil.isBlank(dataStr)) {
            return Result.getResult(JobError.PARAM_CAN_NOT_BE_EMPTY);
        }

        AppConfigDto appConfig = appConfigService.getAppConfig(appName);
        if (appConfig == null) {
            return Result.getResult(JobError.APP_NOT_FOUND);
        }

        Job job = null;
        if (StrUtil.isNotBlank(jobId)) {
            // jobId不为空，从数据库获取已创建的任务
            job = jobMapper.selectById(jobId);
            if (job == null) {
                return Result.getResult(JobError.JOB_NOT_FOUND);
            }
        } else {
            job = new Job();
            job.setJobId(JobIdUtil.newId());
        }

        // 解析param字符串为map对象
        JSONObject reqParamMap = null;
        try {
            reqParamMap = JSON.parseObject(param);
            reqParamMap.put("type", type);
        } catch (Exception e) {
            return Result.getResult(CommonError.PARAM_WRONG);
        }

        // 将reqParam覆盖添加到原有job的param中
        JSONObject params = JSON.parseObject(job.getParam());
        if (params == null) {
            params = new JSONObject();
        }
        params.putAll(reqParamMap);
        job.setParam(JSON.toJSONString(params));
        job.setAppId(appConfig.getAppId());
        job.setMail(mail);
        job.setStatus(JobStatusEnum.WAITING.getKey());
        job.setCreateTime(LocalDateTime.now());
        job.setType(type);

        return jobService.submit(appConfig, dataStr, dataFile, reqParamMap, job);
    }

    @PostMapping("/create")
    public Result createJob(@RequestParam String appName) {
        AppConfigDto appConfig = appConfigService.getAppConfig(appName);
        if (appConfig == null) {
            return Result.getResult(JobError.APP_NOT_FOUND);
        }

        Job job = new Job();
        job.setJobId(JobIdUtil.newId());
        job.setAppId(appConfig.getAppId());
        job.setParam("{}");
        job.setStatus(JobStatusEnum.CREATING.getKey());
        job.setCreateTime(LocalDateTime.now());
        jobMapper.insert(job);
        return Result.success(JobDto.parseJob(job));
    }

    /**
     * 上传文件接口，将文件地址保存至本地，并将路径放入参数中
     */
    @PostMapping("/file/upload")
    public Result uploadFile(@RequestParam String jobId,
                             @RequestParam String fileKey,
                             @RequestParam MultipartFile file) {
        Job job = jobMapper.selectById(jobId);
        if (job == null || !job.getStatus().equals(JobStatusEnum.CREATING.getKey())) {
            return Result.getResult(JobError.JOB_NOT_FOUND);
        }

        AppConfigDto appConfig = AppConfigDto.parse(appConfigService.getById(job.getAppId()));
        JSONObject params = JSON.parseObject(job.getParam());
        try {
            String extName = FileUtil.extName(file.getOriginalFilename());
            if (!StrUtil.isEmpty(extName)) {
                extName = StrUtil.DOT + extName;
            }
            String filePath = appConfig.getRequestPath() + jobId + FileUtil.FILE_SEPARATOR
                    + IdUtil.fastUUID() + extName;
            FileUtil.writeFromStream(file.getInputStream(), filePath);
            params.put(fileKey, filePath);

            job.setParam(JSON.toJSONString(params));
            jobMapper.updateById(job);
            return Result.success(JobDto.parseJob(job));
        } catch (Exception e) {
            e.printStackTrace();
            return Result.fail();
        }
    }

    @PostMapping("/status/update")
    public Result updateJobStatus(@RequestParam String jobId,
                                  @EnumValidation(clazz = JobStatusEnum.class, message = "没有此状态") @RequestParam Integer status,
                                  @RequestParam(required = false) String result) {
        if (status.equals(JobStatusEnum.WAITING.getKey())) {
            return Result.getResult(CommonError.PARAM_WRONG);
        }

        if (status.equals(JobStatusEnum.SUCCESS.getKey())) {
            if (!JSONUtil.isJson(result)) {
                return Result.getResult(CommonError.PARAM_WRONG);
            }
        }

        Job job = jobMapper.selectById(jobId);
        if (job == null) {
            return Result.getResult(CommonError.CONTENT_NOT_FOUND);
        }

        return jobService.updateJobStatus(job, JobStatusEnum.getEnumByKey(status), result);
    }

    @GetMapping("/info/{jobId}")
    public Result getJobInfo(@PathVariable String jobId) {
        return jobService.getJobInfo(jobId);
    }

    @GetMapping("/list")
    public Result getJobList(@RequestParam String appName,
                             @RequestParam(required = false) Integer type,
                             @RequestParam(defaultValue = "true") Boolean filterCreating,
                             @RequestParam(defaultValue = "1") Integer page,
                             @RequestParam(defaultValue = "100") Integer size) {
        AppConfigDto appConfig = appConfigService.getAppConfig(appName);
        if (appConfig == null) {
            return Result.getResult(JobError.APP_NOT_FOUND);
        }

        return jobService.getJobList(appConfig.getAppId(), type, filterCreating, page, size);
    }
}
